

#ifndef VIRAL_MARKETING_H
#define	VIRAL_MARKETING_H

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>

#include "data_structure.h"
#include "b_heap.h"
#include "data_utility.h"

#ifndef WIN32
#include <sys/resource.h>
#endif


#define	MAX_STAGE_NUM	5
#define	MIN_SEED_FILE	"min_seed.txt"
#define	STATISTIC_FILE	"statistic.txt"


//float	prob[4] = {0.25, 0.5, 0.75, 1};


#ifndef WIN32

void GetCurTime( rusage* curTime);

void GetTime( struct rusage* timeStart, struct rusage* timeEnd, float* userTime, float* sysTime);

#endif


extern sta_t sta_v;

/*--------------------------Supporting APIs-----------------------------*/

float prob_weight_gen_IC( );

void prob_weight_adj_LT( graph_t* g_prob_v);

void release_seed_list( s_list_t* s_list_v);

graph_t* construct_graph( config_t* cfg, int mode_n);

void release_graph( graph_t* graph_v);


/*------------------------------Algorithms------------------------------*/
int seed_addition_greedy1( graph_t* g_prob_v, int J, int sample_n, int model_n);

int seed_addition_greedy1_lazy( graph_t* g_prob_v, int J, int sample_n, int model_n);

int seed_addition_greedy2( graph_t* g_prob_v, int J, int sample_n, int model_n);

int seed_addition_greedy2_lazy( graph_t* g_prob_v, int J, int sample_n, int model_n);


/*------------------------Specific for IC-Model-------------------------*/
graph_t* graph_sampling_IC( graph_t* g_prob_v);

float calc_influence_spread_deter_IC( graph_t* g_deter_v, s_list_t* seed_v);

float calc_influence_spread_prob_IC( graph_t* g_prob_v, s_list_t* seed_v, int sample_n);

float calc_influence_gain_on_deter_IC( graph_t* g_deter_v, int* cur_inf, int node_n);

void update_cur_influence_on_deter_IC( graph_t* g_deter_v, int* cur_inf, int node_n);

float seed_addition_greedy_deter_IC( graph_t* g_deter_v, int J);


/*------------------------Specific for LT-Model-------------------------*/
void graph_sampling_LT( graph_t* g_prob_v);

float calc_influence_spread_prob_LT( graph_t* g_deter_v, s_list_t* seed_v, int sample_n);

float calc_influence_spread_deter_LT( graph_t* g_deter_v, s_list_t* seed_v);

float seed_addition_greedy_deter_LT( graph_t* g_deter_v, int J);

/*------------------------------Baselines-------------------------------*/

int random_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n);

int* calc_out_degree( graph_t* g_prob_v);

int degree_oriented_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n);

float calc_centrality_node( graph_t* g_prob_v, int node_n);

float* calc_centrality_graph( graph_t* g_prob_v);

int centrality_oriented_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n);

int page_rank_choose_seed( graph_t* g_prob_v, int J, int sample_n, int model_n);

int opt_brute_force( graph_t* g_prob_v, int J, int sample_n, int model_n);


/*--SCC(strongly connected component--refer to the algorithm book) APIs--*/
void DFS_visit( graph_t* graph_v, int node_n, int &time_cnt, int* finish_t_arr, int scc_cur, int* scc_arr);

int* DFS( graph_t* graph_v);

int DFS_with_order( graph_t* graph_v, int* finish_t_arr, int* scc_arr);

graph_t* transpose_graph( graph_t* graph_v);

scc_t* calc_scc( graph_t* graph_v);

int edge_comp( void* edge1, void* edge2);

edge_s_t* gen_edge_set( graph_t* graph_v, scc_t* scc_v);

graph_t* gen_graph_scc( graph_t* graph_v);

int count_node_zero_in_degree( graph_t* graph_v);

int full_influence_spread_cover( graph_t* graph_v, int sample_n);

/*----------------------------Testing APIs------------------------------*/
void test_greedy_alg( );

#endif